// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang <zhiwei.zhang@saturn.uni-freiburg.de>

#ifndef ZHIWEI_ZHANG_SOURCE_OTHER_ELIASDELTAENCODINGPOINTER_H_
#define ZHIWEI_ZHANG_SOURCE_OTHER_ELIASDELTAENCODINGPOINTER_H_

#include <math.h>
#include <iostream>
#include "../Timer.h"

class EliasDeltaEncodingPointer {
  // we can encode all the natural numbers (except 0) via elias delta encoding.

 public:

  // Encodes the gaps in @list via elias delta encoding.
  template<typename Numeric>
  static void encode(Numeric* plaintext, const size_t& nofElements, bool* encoded,
                       const bool& show = false) {
    Timer t;
    t.start();
    size_t k = 0;
    for (size_t i = 0; i < nofElements; ++i) {
      int prefix = floor(log2(plaintext[i])) + 1;
      int numOfZerosOfPrefix = floor(log2(prefix));
      for (int j = 0; j < numOfZerosOfPrefix; ++j) encoded[k++] = 0;
      for (int j = numOfZerosOfPrefix; j >= 0; --j) {
        encoded[k++] = ((prefix & (1 << j)) != 0);
      }
      for (int j = prefix - 2; j >= 0; --j) {
        encoded[k++] = ((plaintext[i] & (1 << j)) != 0);
      }
    }
    t.stop();
    if (show) {
      std::cout << nofElements << "\t" << k/8 << "\t"
      << t.getUSecs() << "\t";
    }
  }

  // Assumes @bits contains the gaps of a Doclist encoded with elias gamma,
  // Restores the Doclist.
  template<typename Numeric>
  static void decode(bool* bits, const size_t& nofElements, Numeric* decoded,
                         const bool& show = false) {
    Timer t;
    t.start();
    size_t curPosition = 0;
    size_t nofZeros = 0;
    size_t k = 0;
    Numeric num = 0;
    while (k < nofElements) {
      while (bits[curPosition] == 0) ++curPosition;
      nofZeros = curPosition - nofZeros;
      int prefix = -1;
      for (int j = nofZeros; j >= 0; --j) {
        if (bits[curPosition++] == 1) prefix += (1 << j);
      }
      num = (1 << prefix);
      for (int j = prefix - 1; j >= 0; --j) {
        if (bits[curPosition++] == 1) num += (1 << j);
      }
      decoded[k++] = num;
      nofZeros = curPosition;
    }
    t.stop();
    if (show) std::cout << t.getUSecs() << std::endl;
  }
};

#endif  // ZHIWEI_ZHANG_SOURCE_OTHER_ELIASDELTAENCODINGPOINTER_H_
