// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include "../source/Other/EliasDeltaEncodingVector.h"

using std::string;
using std::to_string;
using std::vector;

// _____________________________________________________________________________
TEST(EliasDeltaEncodingVectorTest, encode) {
  vector<int> list;
  string resultToString;
  Bitvector result;

  {
    list.push_back(1);
    EliasDeltaEncodingVector::encode(list, result);
    resultToString += to_string(result[0]);
    ASSERT_EQ("1", resultToString);
  }

  resultToString.clear();
  list.clear();
  result.clear();

  {
    list.push_back(150);
    // as elias delta: 00010000010110
    EliasDeltaEncodingVector::encode(list, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("00010000010110", resultToString);
  }

  resultToString.clear();
  list.clear();
  result.clear();

  {
    list.push_back(1);
    list.push_back(150);
    // Gaps 1 and 150 like before
    EliasDeltaEncodingVector::encode(list, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("100010000010110", resultToString);
  }
}

// _____________________________________________________________________________
TEST(EliasDeltaEncodingVectorTest, encodeDecode) {
  vector<int> plaintext;
  Bitvector encoded;
  vector<int> decoded;

  plaintext.push_back(1);
  plaintext.push_back(5);
  plaintext.push_back(16400);
  plaintext.push_back(20000);
  plaintext.push_back(70000);
  decoded.reserve(5);

  EliasDeltaEncodingVector::encode(plaintext, encoded);
  EliasDeltaEncodingVector::decode(encoded, decoded);
  ASSERT_EQ(plaintext.size(), decoded.size());
  for (size_t i = 0; i < plaintext.size(); ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }
}
