// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include "../source/Other/GolombEncodingVector.h"

using std::string;
using std::to_string;
using std::vector;

// _____________________________________________________________________________
TEST(GolombEncodingVectorTest, encode) {
  vector<int> list;
  int M = 16;
  Bitvector result;
  string resultToString;

  {
    list.push_back(1);
    GolombEncodingVector::encode(list, M, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("10001", resultToString);
  }

  resultToString.clear();
  result.clear();
  list.clear();

  {
    list.push_back(150);
    // as golomb: 00000000010110
    GolombEncodingVector::encode(list, M, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("00000000010110", resultToString);
  }

  resultToString.clear();
  result.clear();
  list.clear();

  {
    list.push_back(1);
    list.push_back(150);
    // Gaps 1 and 150 like before
    GolombEncodingVector::encode(list, M, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("1000100000000010110", resultToString);
  }
}

// _____________________________________________________________________________
TEST(GolombEncodingVectorTest, encodeDecode) {
  vector<int> plaintext;
  int M = 16;
  vector<int> decoded;
  Bitvector encoded;

  plaintext.push_back(1);
  plaintext.push_back(5);
  plaintext.push_back(16400);
  plaintext.push_back(20000);
  plaintext.push_back(70000);

  GolombEncodingVector::encode(plaintext, M, encoded);
  GolombEncodingVector::decode(encoded, M, decoded);
  ASSERT_EQ(plaintext.size(), decoded.size());
  for (size_t i = 0; i < plaintext.size(); ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }
}
