// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang
#include <random>
#include <math.h>
#include <gtest/gtest.h>
#include "../source/Simple8b/Simple8bCodeClass.h"
#include "../source/GenerateNumber.h"

// _____________________________________________________________________________
TEST(Simple8bCodeClassTest, getNextWord) {
  int* test = new int[661]();
  test[660] = 1;
  ad_utility_class::Simple8bCode s8bc(test, 661);
  uint64_t result;

  result = s8bc.getNextWord();
  ASSERT_EQ(0, result);

  result = s8bc.getNextWord();
  ASSERT_EQ(0, result);

  result = s8bc.getNextWord();
  ASSERT_EQ(0, result);

  result = s8bc.getNextWord();
  ASSERT_EQ(1, result);

  delete[] test;
}

// _____________________________________________________________________________
TEST(Simple8bCodeClassTest, successor) {
  int* test = new int[5];
  for (size_t i = 0; i < 5; ++i) {
    test[i] = pow(i, 2);
  }
  ad_utility_class::Simple8bCode s8bc(test, 5);
  int initial = 0;

  int* result = &initial;
  s8bc.successor(result);
  ASSERT_TRUE(*result == 0);

  s8bc.successor(result);
  ASSERT_TRUE(1 == *result);

  s8bc.successor(result);
  ASSERT_TRUE(4 == *result);

  s8bc.successor(result);
  ASSERT_TRUE(9 == *result);

  s8bc.successor(result);
  ASSERT_TRUE(16 == *result);

  delete[] test;
  delete result;
}

// _____________________________________________________________________________
TEST(Simple8bCodeClassTest, forwardSearch) {
  int* test = new int[60];
  for (size_t i = 0; i < 60; ++i) {
    test[i] = pow(i, 2);
  }
  ad_utility_class::Simple8bCode s8bc(test, 60);
  int initial = 0;

  s8bc.forwardSearch(40);
  int* result = &initial;
  s8bc.successor(result);
  ASSERT_TRUE(49 == *result);
  

  s8bc.forwardSearch(81);
  s8bc.successor(result);
  ASSERT_TRUE(81 == *result);

  s8bc.forwardSearch(145);
  s8bc.successor(result);
  ASSERT_TRUE(169 == *result);

  s8bc.forwardSearch(2499);
  s8bc.successor(result);
  ASSERT_TRUE(2500 == *result);

  delete[] test;
  delete result;
}

// _____________________________________________________________________________
TEST(Simple8bCodeClassTest, forwardSeek) {
  int* test = new int[60];
  for (size_t i = 0; i < 60; ++i) {
    test[i] = pow(i, 2);
  }
  ad_utility_class::Simple8bCode s8bc(test, 60);
  int initial = 0;

  s8bc.forwardSeek(9);
  int* result = &initial;
  s8bc.successor(result);
  ASSERT_TRUE(81 == *result);

  s8bc.forwardSeek(10);
  s8bc.successor(result);
  ASSERT_TRUE(400 == *result);

  s8bc.forwardSeek(9);
  s8bc.successor(result);
  ASSERT_TRUE(900 == *result);

  s8bc.forwardSeek(9);
  s8bc.successor(result);
  ASSERT_TRUE(1600 == *result);

  
  delete[] test;
  delete result;
}
